import _regeneratorRuntime from "@babel/runtime/regenerator";
import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
import _createClass from "@babel/runtime/helpers/esm/createClass";
import _inheritsLoose from "@babel/runtime/helpers/esm/inheritsLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
import React, { PureComponent, Component, createContext, memo, useContext } from 'react';
import { isValidElementType } from 'react-is';
import shallowequal from 'shallowequal';
import SSRContext from './SSRContext';
var fetchDefaultOptions = {
  // fetcher name
  name: 'data',
  // we may provide options for the providers
  providerOptions: null,
  // we may provide options for the consumers
  consumerOptions: null
};
var providerDefaultOptions = {
  // variables (function or object)
  variables: null,
  // pure option
  pure: true,
  // should we refetch on props change
  refetchOnPropChanges: false,
  // function defining if there's a change on props
  shouldUpdate: shallowequal,
  // we may want to process the fetched data
  cleanData: null
};
var consumerDefaultOptions = {
  // pure option
  pure: true
};

var getDisplayName = function getDisplayName(WrappedComponent) {
  return WrappedComponent.displayName || WrappedComponent.name || 'Component';
};

var createDataFetcher = function createDataFetcher(fetcher, customFetcherOptions) {
  var _ref;

  if (customFetcherOptions === void 0) {
    customFetcherOptions = null;
  }

  // create final options
  var options = _extends({}, fetchDefaultOptions, customFetcherOptions); // capitalize the name to use it as display name


  var displayName = options.name.charAt(0).toUpperCase() + options.name.substr(1); // create a context for this data fetcher

  var Context = createContext(null); // we build a custom hook to consume the context

  var useData = function useData() {
    return useContext(Context);
  }; // we build the HOC to fetch data and provide the context


  var withDataFetcher = function withDataFetcher(providerCusomOptions) {
    // initialize the options with the default options
    var providerOptions = _extends({}, providerDefaultOptions, options.providerOptions); // here is our factory to build the HOC provider


    var getHOC = function getHOC(WrappedComponent) {
      var ComponentBase = options.pure ? PureComponent : Component;

      var Provider = /*#__PURE__*/function (_ComponentBase) {
        _inheritsLoose(Provider, _ComponentBase);

        function Provider(props) {
          var _this;

          _this = _ComponentBase.call(this, props) || this; // pending promise to fetch data

          _this.refetch = function (customVariables) {
            if (customVariables === void 0) {
              customVariables = null;
            }

            var variables = _extends({}, _this.getVariables(), customVariables); // fetch again


            _this.fetchPromise = _this.fetch(variables, true); // and force the update

            _this.forceUpdate();

            return _this.fetchPromise;
          };

          _this.fetchPromise = null; // previous props & state

          _this.prevProps = null; // current context

          _this.contextValue = null; // SSR index

          _this.index = null;
          return _this;
        }

        var _proto = Provider.prototype;

        _proto.getVariables = function getVariables() {
          return providerOptions.variables instanceof Function ? providerOptions.variables(this.props) : providerOptions.variables;
        };

        _proto.fetch = /*#__PURE__*/function () {
          var _fetch = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(variables) {
            var result, data, cleanedData;
            return _regeneratorRuntime.wrap(function _callee$(_context) {
              while (1) {
                switch (_context.prev = _context.next) {
                  case 0:
                    result = {
                      err: null,
                      data: null
                    };
                    _context.prev = 1;
                    // inform we're fetching
                    this.updateContext({
                      loading: true
                    }); // update the state at first

                    _context.next = 5;
                    return fetcher(variables, this.props);

                  case 5:
                    data = _context.sent;
                    cleanedData = providerOptions.cleanData ? providerOptions.cleanData(data, variables, this.props) : data; // update the state with the data

                    this.updateContext({
                      loading: false,
                      data: cleanedData,
                      error: null
                    }); // do an update just after fetching

                    this.forceUpdate(); // update the result

                    result.data = cleanedData;
                    _context.next = 17;
                    break;

                  case 12:
                    _context.prev = 12;
                    _context.t0 = _context["catch"](1);
                    // update the state with the error
                    this.updateContext({
                      loading: false,
                      data: null,
                      error: _context.t0
                    }); // do an update after getting an error

                    this.forceUpdate(); // update the result

                    result.error = _context.t0;

                  case 17:
                    _context.prev = 17;
                    this.fetchPromise = null;
                    return _context.finish(17);

                  case 20:
                    return _context.abrupt("return", result);

                  case 21:
                  case "end":
                    return _context.stop();
                }
              }
            }, _callee, this, [[1, 12, 17, 20]]);
          }));

          function fetch(_x) {
            return _fetch.apply(this, arguments);
          }

          return fetch;
        }();

        _proto.updateContext = function updateContext(newContext) {
          // the context
          this.contextValue = _extends({
            // initial values
            error: null,
            data: null
          }, this.contextValue, newContext, {
            // and ensure the right refetch method is given
            refetch: this.refetch
          });
        };

        _proto.render = function render() {
          var _this$context = this.context,
              ssr = _this$context.ssr,
              renderer = _this$context.renderer;

          if (renderer) {
            this.index = renderer.register();

            if (renderer.hasState(this.index)) {
              var _renderer$getState = renderer.getState(this.index),
                  result = _renderer$getState.result,
                  props = _renderer$getState.props;

              this.updateContext(_extends({
                loading: false
              }, result));
              this.prevProps = props;
            }
          }

          if (this.shouldFetch) {
            // compute variables from props
            var variables = _extends({}, this.getVariables()); // and fetch


            this.fetchPromise = this.fetch(variables); // remember the props we used

            this.prevProps = this.props;

            if (ssr) {
              // register our fetch to be processed on SSR
              renderer.attachPromise(this.fetchPromise, this.props, this.index);
            }
          }

          return /*#__PURE__*/React.createElement(Context.Provider, {
            value: this.contextValue
          }, /*#__PURE__*/React.createElement(WrappedComponent, _extends({}, this.contextValue, this.props)));
        };

        _createClass(Provider, [{
          key: "shouldFetch",
          get: function get() {
            if (this.contextValue === null && this.fetchPromise === null) {
              // first fetch
              return true;
            }

            if (this.prevProps === this.props || this.fetchPromise) {
              // props are the same or it's already fetching
              return false;
            } // only fetch if we're looking to fetch on prop updates


            return providerOptions.refetchOnPropChanges && !providerOptions.shouldUpdate(this.props, this.prevProps);
          }
        }]);

        return Provider;
      }(ComponentBase);

      Provider.contextType = SSRContext;
      var wrappedDisplayName = getDisplayName(WrappedComponent);
      Provider.displayName = "with" + displayName + "Fetcher(" + wrappedDisplayName + ")";
      return Provider;
    };

    if (isValidElementType(providerCusomOptions)) {
      // the argument is a valid react element type, so let's consider there's no options
      return getHOC(providerCusomOptions);
    } // we do have custom options, let's merge those


    providerOptions = _extends({}, providerOptions, providerCusomOptions); // and return the factory

    return getHOC;
  }; // then the HOC to consume data


  var withData = function withData(consumerCustomOptions) {
    // initialize the options with the default options
    var customerOptions = _extends({}, consumerDefaultOptions, options.consumerOptions); // here is our factory to build the HOC consumer


    var getHOC = function getHOC(WrappedComponent) {
      var Consumer = function Consumer(props) {
        var contextValue = useData();
        return /*#__PURE__*/React.createElement(WrappedComponent, _extends({}, contextValue, props));
      };

      if (customerOptions.pure) {
        Consumer = memo(Consumer);
      }

      var wrappedDisplayName = getDisplayName(WrappedComponent);
      Consumer.displayName = "with" + displayName + "(" + wrappedDisplayName + ")";
      return Consumer;
    };

    if (isValidElementType(consumerCustomOptions)) {
      // the argument is a valid react element type, so let's consider there's no options
      return getHOC(consumerCustomOptions);
    } // we do have custom options, let's merge those


    customerOptions = _extends({}, customerOptions, consumerCustomOptions); // and return the factory

    return getHOC;
  };

  return _ref = {}, _ref["use" + displayName] = useData, _ref["with" + displayName + "Fetcher"] = withDataFetcher, _ref["with" + displayName] = withData, _ref;
};

export default createDataFetcher;