"use strict";

exports.__esModule = true;
exports["default"] = void 0;

var SSRRenderer = /*#__PURE__*/function () {
  function SSRRenderer(initialState) {
    if (initialState === void 0) {
      initialState = [];
    }

    // list of promises to wait for
    this.promises = []; // persisted data through multiple run for SSR

    this.state = initialState; // current index

    this.index = 0; // has the state been consumed

    this.consumed = false;
  }

  var _proto = SSRRenderer.prototype;

  _proto.hasPromises = function hasPromises() {
    return this.promises.length > 0;
  };

  _proto.hasState = function hasState(index) {
    return !this.consumed && index + 1 <= this.state.length;
  };

  _proto.register = function register() {
    var index = this.index;
    this.index += 1;
    return index;
  };

  _proto.getState = function getState(index) {
    var state = this.state[index];

    if (index + 1 === this.state.length) {
      this.consumed = true;
    }

    return state;
  };

  _proto.attachPromise = function attachPromise(promise, props, index) {
    var _this = this;

    this.promises.push(promise.then(function (result) {
      // register everything for this promise
      _this.state[index] = {
        result: result,
        props: props
      }; // and return the result again

      return result;
    }));
  };

  _proto.consume = function consume() {
    // get promises to wait for
    var promises = this.promises; // reset the renderer

    this.index = 0;
    this.promises = [];
    this.consumed = false;
    return Promise.all(promises);
  };

  return SSRRenderer;
}();

exports["default"] = SSRRenderer;